@@ -49,11 +49,13 @@ class Manifest(AttributeManager):
         self.specDigest = None
         self.repositories = {}
 
-    def _setSpec(self, config, projectRepositories=None):
+    def _setSpec(self, config):
         """
         Set the TOSCA service template.
         """
-        repositories = self.updateRepositories(config, projectRepositories or [])
+        repositories = {
+            name: repo.repository.tpl for name, repo in self.repositories.items()
+        }
         spec = config.get("spec", {})
         self.tosca = self._loadSpec(spec, self.path, repositories)
         self.specDigest = self.getSpecDigest(spec)
@@ -90,9 +92,7 @@ class Manifest(AttributeManager):
         ):
             # note: we only recorded the baseDir not the name of the included file
             path = toscaDef.baseDir
-        return ToscaSpec(
-            toscaDef, spec.get("inputs"), spec, path, self.getImportResolver()
-        )
+        return ToscaSpec(toscaDef, spec, path, self.getImportResolver())
 
     def getSpecDigest(self, spec):
         m = hashlib.sha1()  # use same digest function as git
@@ -172,49 +172,50 @@ class Manifest(AttributeManager):
                 ]
         return resourceChanges
 
-    # XXX unused
-    # def loadConfigChange(self, changeId):
-    #     """
-    # Reconstruct the Configuration that was applied in the past
-    # """
-    #     changeSet = self.changeSets.get(changeId)
-    #     if not changeSet:
-    #         raise UnfurlError("can not find changeset for changeid %s" % changeId)
-    #
-    #     configChange = ConfigChange()
-    #     Manifest.loadStatus(changeSet, configChange)
-    #     configChange.changeId = changeSet.get("changeId", 0)
-    #     configChange.parentId = changeSet.get("parentId")
-    #
-    #     configChange.inputs = changeSet.get("inputs")
-    #
-    #     configChange.dependencies = {}
-    #     for val in changeSet.get("dependencies", []):
-    #         key = val.get("name") or val["ref"]
-    #         assert key not in configChange.dependencies
-    #         configChange.dependencies[key] = Dependency(
-    #             val["ref"],
-    #             val.get("expected"),
-    #             val.get("schema"),
-    #             val.get("name"),
-    #             val.get("required"),
-    #             val.get("wantList", False),
-    #         )
-    #
-    #     if "changes" in changeSet:
-    #         configChange.resourceChanges = self.loadResourceChanges(
-    #             changeSet["changes"]
-    #         )
-    #
-    #     configChange.result = changeSet.get("result")
-    #     configChange.messages = changeSet.get("messages", [])
-    #
-    #     # XXX
-    #     # ('action', ''),
-    #     # ('target', ''), # nodeinstance key
-    #     # implementationType: configurator resource | artifact | configurator class
-    #     # implementation: repo:key#commitid | className:version
-    #     return configChange
+    def loadConfigChange(self, changeId):
+        """
+        Reconstruct the Configuration that was applied in the past
+        """
+        from .configurator import Dependency
+
+        changeSet = self.changeSets.get(changeId)
+        if not changeSet:
+            raise UnfurlError("can not find changeset for changeid %s" % changeId)
+
+        configChange = ConfigChange()
+        Manifest.loadStatus(changeSet, configChange)
+        configChange.changeId = changeSet.get("changeId", 0)
+        configChange.parentId = changeSet.get("parentId")
+
+        configChange.inputs = changeSet.get("inputs")
+
+        configChange.dependencies = {}
+        for val in changeSet.get("dependencies", []):
+            key = val.get("name") or val["ref"]
+            assert key not in configChange.dependencies
+            configChange.dependencies[key] = Dependency(
+                val["ref"],
+                val.get("expected"),
+                val.get("schema"),
+                val.get("name"),
+                val.get("required"),
+                val.get("wantList", False),
+            )
+
+        if "changes" in changeSet:
+            configChange.resourceChanges = self.loadResourceChanges(
+                changeSet["changes"]
+            )
+
+        configChange.result = changeSet.get("result")
+        configChange.messages = changeSet.get("messages", [])
+
+        # XXX
+        # ('action', ''),
+        # ('target', ''), # nodeinstance key
+        # implementationType: configurator resource | artifact | configurator class
+        # implementation: repo:key#commitid | className:version
+        return configChange
 
     # XXX this is only used by commented out Taskview.createConfigurationSpec()
     # def loadConfigSpec(self, configName, spec):
@@ -244,27 +245,26 @@ class Manifest(AttributeManager):
                 raise UnfurlError("can not find parent instance %s" % pname)
 
         resource = self._createEntityInstance(NodeInstance, rname, resourceSpec, parent)
+        if resourceSpec.get("capabilities"):
+            for key, val in resourceSpec["capabilities"].items():
+                self._createEntityInstance(CapabilityInstance, key, val, resource)
 
-        resource._capabilities = []
-        for key, val in resourceSpec.get("capabilities", {}).items():
-            self._createEntityInstance(CapabilityInstance, key, val, resource)
-
-        resource._requirements = []
-        for key, val in resourceSpec.get("requirements", {}).items():
-            # parent will be the capability, should have already been created
-            capabilityId = val.get("capability")
-            if not capabilityId:
-                raise UnfurlError("requirement is missing capability %s" % key)
-            capability = capabilityId and self.getRootResource().query(capabilityId)
-            if not capability or not isinstance(capability, CapabilityInstance):
-                raise UnfurlError("can not find capability %s" % capabilityId)
-            if capability._relationships is None:
-                capability._relationships = []
-            requirement = self._createEntityInstance(
-                RelationshipInstance, key, val, capability
-            )
-            requirement.source = resource
-            resource.requirements.append(requirement)
+        if resourceSpec.get("requirements"):
+            for key, val in resourceSpec["requirements"].items():
+                # parent will be the capability, should have already been created
+                capabilityId = val.get("capability")
+                if not capabilityId:
+                    raise UnfurlError("requirement is missing capability %s" % key)
+                capability = capabilityId and self.getRootResource().query(capabilityId)
+                if not capability or not isinstance(capability, CapabilityInstance):
+                    raise UnfurlError("can not find capability %s" % capabilityId)
+                if capability._relationships is None:
+                    capability._relationships = []
+                requirement = self._createEntityInstance(
+                    RelationshipInstance, key, val, capability
+                )
+                requirement.source = resource
+                resource.requirements.append(requirement)
 
         for key, val in resourceSpec.get("instances", {}).items():
             self.createNodeInstance(key, val, resource)
@@ -365,16 +365,17 @@ class Manifest(AttributeManager):
         self.repositories[repository.name] = repository
         return repository
 
-    def updateRepositories(self, config, inlineRepositories):
+    def updateRepositories(self, config, inlineRepositories=None):
         repositories = self._getRepositories(config)
         for name, tpl in repositories.items():
             if name not in self.repositories:
                 toscaRepository = Repository(name, tpl)
                 self.repositories[name] = RepoView(toscaRepository, None)
-        for repository in inlineRepositories:
-            if isinstance(repository, dict):
-                repository = Repository(repository.pop("name"), repository)
-            self.addRepository(None, repository, "")
+        if inlineRepositories:
+            for repository in inlineRepositories:
+                if isinstance(repository, dict):
+                    repository = Repository(repository.pop("name"), repository)
+                self.addRepository(None, repository, "")
         return self._setRepositories()
 
     @staticmethod
@@ -434,7 +435,7 @@ class Manifest(AttributeManager):
                 repositories["spec"] = repository
             else:
                 repositories["spec"] = repositories["self"]
-        return {name: repo.repository.tpl for name, repo in repositories.items()}
+        return {name: repo.repository.tpl for name, repo in self.repositories.items()}
 
     def loadYamlInclude(
         self, yamlConfig, templatePath, baseDir, warnWhenNotFound=False
@@ -485,6 +486,7 @@ class SnapShotManifest(Manifest):
         self.manifest = YamlConfig(
             oldManifest, manifest.path, loadHook=self.loadYamlInclude
         )
+        self.updateRepositories(oldManifest)
         expanded = self.manifest.expanded
         # just needs the spec, not root resource
         self._setSpec(expanded)
