@@ -4,8 +4,10 @@
 TOSCA implementation
 """
 from .tosca_plugins import TOSCA_VERSION
-from .util import UnfurlValidationError, getBaseDir
-from .eval import Ref
+from .util import UnfurlError, UnfurlValidationError, getBaseDir
+from .eval import Ref, RefContext
+from .result import ResourceRef, ResultsList
+from .merge import patchDict
 from toscaparser.tosca_template import ToscaTemplate
 from toscaparser.properties import Property
 from toscaparser.elements.entity_type import EntityType
@@ -13,7 +15,7 @@ from toscaparser.elements.statefulentitytype import StatefulEntityType
 import toscaparser.workflow
 import toscaparser.imports
 import toscaparser.artifacts
-from toscaparser.common.exception import ExceptionCollector, ValidationError
+from toscaparser.common.exception import ExceptionCollector
 import six
 import logging
 from ruamel.yaml.comments import CommentedMap
@@ -70,35 +72,44 @@ class ToscaSpec(object):
     ConfiguratorType = "unfurl.nodes.Configurator"
     InstallerType = "unfurl.nodes.Installer"
 
-    def __init__(self, toscaDef, inputs=None, instances=None, path=None, resolver=None):
-        self.discovered = None
-        if isinstance(toscaDef, ToscaTemplate):
-            self.template = toscaDef
-        else:
-            topology_tpl = toscaDef.get("topology_template")
-            if not topology_tpl:
-                toscaDef["topology_template"] = dict(
-                    node_templates={}, relationship_templates={}
-                )
+    def _overlay(self, overlays):
+        def _findMatches():
+            ExceptionCollector.start()  # clears previous errors
+            for expression, _tpl in overlays.items():
+                try:
+                    match = Ref(expression).resolveOne(
+                        RefContext(self.topology, trace=0)
+                    )
+                    if not match:
+                        continue
+                    if isinstance(match, ResultsList):
+                        for item in match:
+                            yield (item, _tpl)
+                    else:
+                        yield (match, _tpl)
+                except:
+                    ExceptionCollector.appendException(
+                        UnfurlValidationError(
+                            'error evaluating decorator match expression "%s"'
+                            % expression,
+                            True,
+                        )
+                    )
 
-            if instances:
-                self.loadInstances(toscaDef, instances)
+        matches = list(_findMatches())
+        return [
+            patchDict(m[0].toscaEntityTemplate.entity_tpl, m[1], True) for m in matches
+        ]
 
-            logger.info("Validating TOSCA template at %s", path)
-            try:
-                # need to set a path for the import loader
-                self.template = ToscaTemplate(
-                    path=path,
-                    parsed_params=inputs,
-                    yaml_dict_tpl=toscaDef,
-                    import_resolver=resolver,
-                )
-            except ValidationError:
-                message = "\n".join(ExceptionCollector.getExceptionsReport(False))
-                raise UnfurlValidationError(
-                    "TOSCA validation failed for %s: \n%s" % (path, message),
-                    ExceptionCollector.getExceptions(),
-                )
+    def _parseTemplate(self, path, inputs, toscaDef, resolver):
+        # need to set a path for the import loader
+        self.template = ToscaTemplate(
+            path=path,
+            parsed_params=inputs,
+            yaml_dict_tpl=toscaDef,
+            import_resolver=resolver,
+            verify=False,
+        )
 
         self.nodeTemplates = {}
         self.installers = {}
@@ -108,21 +119,85 @@ class ToscaSpec(object):
             if template.is_derived_from(self.InstallerType):
                 self.installers[template.name] = nodeTemplate
             self.nodeTemplates[template.name] = nodeTemplate
-
         if hasattr(self.template, "relationship_templates"):
             # user-declared RelationshipTemplates, source and target will be None
             for template in self.template.relationship_templates:
                 relTemplate = RelationshipSpec(template, self)
                 self.relationshipTemplates[template.name] = relTemplate
-
+        self.loadImportedDefaultTemplates()
         self.topology = TopologySpec(self, inputs)
-        self.load_workflows()
+        self.loadWorkflows()
         self.groups = {
             g.name: GroupSpec(g, self) for g in self.template.topology_template.groups
         }
+        self.policies = {
+            p.name: PolicySpec(p, self)
+            for p in self.template.topology_template.policies
+        }
+
+    def __init__(
+        self,
+        toscaDef,
+        spec=None,
+        path=None,
+        resolver=None,
+    ):
+        self.discovered = None
+        if spec:
+            inputs = spec.get("inputs")
+        else:
+            inputs = None
+
+        if isinstance(toscaDef, ToscaTemplate):
+            self.template = toscaDef
+        else:
+            topology_tpl = toscaDef.get("topology_template")
+            if not topology_tpl:
+                toscaDef["topology_template"] = dict(
+                    node_templates={}, relationship_templates={}
+                )
+
+            if spec:
+                self.loadInstances(toscaDef, spec)
+
+            logger.info("Validating TOSCA template at %s", path)
+            self._parseTemplate(path, inputs, toscaDef, resolver)
+            decorators = self.loadDecorators()
+            if decorators:
+                # copy errors before we clear them in _overlay
+                errorsSoFar = ExceptionCollector.exceptions[:]
+                self._overlay(decorators)
+                if ExceptionCollector.exceptionsCaught():
+                    # abort if overlay caused errors
+                    # report previously collected errors too
+                    ExceptionCollector.exceptions[:0] = errorsSoFar
+                    message = "\n".join(
+                        ExceptionCollector.getExceptionsReport(
+                            logger.getEffectiveLevel() < logging.INFO
+                        )
+                    )
+                    raise UnfurlValidationError(
+                        "TOSCA validation failed for %s: \n%s" % (path, message),
+                        ExceptionCollector.getExceptions(),
+                    )
+                # overlay modifies tosaDef in-place, try reparsing it
+                self._parseTemplate(path, inputs, toscaDef, resolver)
+
+            if ExceptionCollector.exceptionsCaught():
+                message = "\n".join(
+                    ExceptionCollector.getExceptionsReport(
+                        logger.getEffectiveLevel() < logging.INFO
+                    )
+                )
+                raise UnfurlValidationError(
+                    "TOSCA validation failed for %s: \n%s" % (path, message),
+                    ExceptionCollector.getExceptions(),
+                )
 
     @property
     def baseDir(self):
+        if self.template.path is None:
+            return None
         return getBaseDir(self.template.path)
 
     def _getProjectDir(self):
@@ -142,21 +217,35 @@ class ToscaSpec(object):
         self.discovered[name] = tpl
         return nodeSpec
 
-    def load_workflows(self):
+    def loadDecorators(self):
+        decorators = {}
+        for import_tpl in self.template.nested_tosca_tpls.values():
+            decorators.update(import_tpl.get("decorators") or {})
+        decorators.update(self.template.tpl.get("decorators") or {})
+        return decorators
+
+    def loadImportedDefaultTemplates(self):
+        decorators = {}
+        for topology in self.template.nested_topologies.values():
+            for nodeTemplate in topology.nodetemplates:
+                if (
+                    "default" in nodeTemplate.directives
+                    and nodeTemplate.name not in self.nodeTemplates
+                ):
+                    nodeSpec = NodeSpec(nodeTemplate, self)
+                    self.nodeTemplates[nodeSpec.name] = nodeSpec
+        return decorators
+
+    def loadWorkflows(self):
         # we want to let different types defining standard workflows like deploy
         # so we need to support importing workflows
         workflows = {
             name: [Workflow(w)]
             for name, w in self.template.topology_template.workflows.items()
         }
-        for import_tpl in self.template.nested_tosca_tpls.values():
-            importedWorkflows = import_tpl.get("topology_template", {}).get("workflows")
-            if importedWorkflows:
-                for name, val in importedWorkflows.items():
-                    workflows.setdefault(name, []).append(
-                        Workflow(toscaparser.workflow.Workflow(name, val))
-                    )
-
+        for topology in self.template.nested_topologies.values():
+            for name, w in topology.workflows.items():
+                workflows.setdefault(name, []).append(Workflow(w))
         self._workflows = workflows
 
     def getWorkflow(self, workflow):
@@ -319,7 +408,7 @@ def findProps(attributes, attributeDefs, matchfn):
 
 
 # represents a node, capability or relationship
-class EntitySpec(object):
+class EntitySpec(ResourceRef):
     # XXX need to define __eq__ for spec changes
     def __init__(self, toscaNodeTemplate, spec=None):
         self.toscaEntityTemplate = toscaNodeTemplate
@@ -359,9 +448,13 @@ class EntitySpec(object):
         else:
             self.defaultAttributes = {}
 
+    parent = None
+
     def __reflookup__(self, key):
         """Make attributes available to expressions"""
-        if key in ["name", "type", "uri", "groups"]:
+        if key[0] == ".":
+            return self._getProp(key)
+        if key in ["name", "type", "uri", "groups", "policies"]:
             return getattr(self, key)
         raise KeyError(key)
 
@@ -376,6 +469,10 @@ class EntitySpec(object):
             if self.name in g.members:
                 yield g
 
+    @property
+    def policies(self):
+        return []
+
     def isCompatibleTarget(self, targetStr):
         if self.name == targetStr:
             return True
@@ -443,9 +540,18 @@ class NodeSpec(EntitySpec):
         EntitySpec.__init__(self, template, spec)
         self._capabilities = None
         self._requirements = None
-        self._relationships = None
+        self._relationships = []
         self._artifacts = None
 
+    def __reflookup__(self, key):
+        try:
+            return super(NodeSpec, self).__reflookup__(key)
+        except KeyError:
+            relationship = self.getRelationship(key)
+            if not relationship:
+                raise KeyError(key)
+            return relationship
+
     @property
     def artifacts(self):
         if self._artifacts is None:
@@ -455,15 +561,26 @@ class NodeSpec(EntitySpec):
             }
         return self._artifacts
 
+    @property
+    def policies(self):
+        if not self.spec:
+            return
+        for p in self.spec.policies.values():
+            if p.toscaEntityTemplate.targets_type == "groups":
+                # the policy has groups as members, see if this node's groups is one of them
+                if p.members & set(g.name for g in self.groups):
+                    yield p
+            elif p.toscaEntityTemplate.targets_type == "node_templates":
+                if self.name in p.members:
+                    yield p
+
     @property
     def requirements(self):
         if self._requirements is None:
             self._requirements = {}
             nodeTemplate = self.toscaEntityTemplate
-            for req in nodeTemplate.requirements:
+            for (relTpl, req, reqDef) in nodeTemplate.relationships:
                 name, values = next(iter(req.items()))
-                # _get_explicit_relationship should add this the target's relationship_tpl if needed
-                reqDef, relTpl = nodeTemplate._get_explicit_relationship(req)
                 reqSpec = RequirementSpec(name, reqDef, self)
                 if relTpl.target:
                     nodeSpec = self.spec.getTemplate(relTpl.target.name)
@@ -493,12 +610,12 @@ class NodeSpec(EntitySpec):
         return self._getRelationshipSpecs()
 
     def _getRelationshipSpecs(self):
-        if self._relationships is None:
+        if len(self._relationships) != len(self.toscaEntityTemplate.relationship_tpl):
             # relationship_tpl is a list of RelationshipTemplates that target the node
-            self._relationships = [
-                RelationshipSpec(r, self.spec)
-                for r in self.toscaEntityTemplate.relationship_tpl
-            ]
+            rIds = set(id(r.toscaEntityTemplate) for r in self._relationships)
+            for r in self.toscaEntityTemplate.relationship_tpl:
+                if id(r) not in rIds:
+                    self._relationships.append(RelationshipSpec(r, self.spec, self))
         return self._relationships
 
     def getCapabilityInterfaces(self):
@@ -523,7 +640,6 @@ class NodeSpec(EntitySpec):
 
     def addRelationship(self, reqSpec):
         # find the relationship for this requirement:
-        self._relationships = None
         for relSpec in self._getRelationshipSpecs():
             # the RelationshipTemplate should have had the source node assigned by the tosca parser
             # XXX this won't distinguish between more than one relationship between the same two nodes
@@ -544,23 +660,6 @@ class NodeSpec(EntitySpec):
             )
             raise UnfurlValidationError(msg)
 
-        # figure out which capability the relationship targets:
-        for capability in self.capabilities.values():
-            if reqSpec.isCapable(capability):
-                assert reqSpec.relationship
-                assert (
-                    not reqSpec.relationship.capability
-                    or reqSpec.relationship.capability is capability
-                )
-                reqSpec.relationship.capability = capability
-                break
-        else:
-            capabilities = [c.name for c in self.capabilities.values()]
-            raise UnfurlValidationError(
-                'capabilities %s not found for requirement "%s" on node "%s"'
-                % (capabilities, reqSpec.name, self.name)
-            )
-
     @property
     def abstract(self):
         for name in ("select", "substitute"):
@@ -578,9 +677,10 @@ class RelationshipSpec(EntitySpec):
     Links a RequirementSpec to a CapabilitySpec.
     """
 
-    def __init__(self, template=None, spec=None):
+    def __init__(self, template=None, spec=None, targetNode=None):
         # template is a RelationshipTemplate
         # It is a full-fledged entity with a name, type, properties, attributes, interfaces, and metadata.
+        # its connected through target, source, capability
         # its RelationshipType has valid_target_types
         if not template:
             template = _defaultTopology.topology_template.relationship_templates[0]
@@ -590,6 +690,14 @@ class RelationshipSpec(EntitySpec):
         EntitySpec.__init__(self, template, spec)
         self.requirement = None
         self.capability = None
+        if targetNode:
+            assert targetNode.toscaEntityTemplate is template.target
+            for c in targetNode.capabilities.values():
+                if c.toscaEntityTemplate is template.capability:
+                    self.capability = c
+                    break
+            else:
+                raise UnfurlError('capability %s not found in %s for %s' % (template.capability.name, [c.name for c in targetNode.capabilities.values()], targetNode.name))
 
     @property
     def source(self):
@@ -599,6 +707,17 @@ class RelationshipSpec(EntitySpec):
     def target(self):
         return self.capability.parentNode if self.capability else None
 
+    def __reflookup__(self, key):
+        try:
+            return super(RelationshipSpec, self).__reflookup__(key)
+        except KeyError:
+            if self.capability:
+                if self.capability.parentNode.isCompatibleTarget(key):
+                    return self.capability.parentNode
+                if self.capability.isCompatibleTarget(key):
+                    return self.capability
+            raise KeyError(key)
+
     def getUri(self):
         suffix = "~r~" + self.name
         return self.source.name + suffix if self.source else suffix
@@ -650,19 +769,6 @@ class RequirementSpec(object):
     def getInterfaces(self):
         return self.relationship.getInterfaces() if self.relationship else []
 
-    def isCapable(self, capability):
-        # XXX consider self.requirements_tpl.node, node_filter
-        capabilityName = self.requirements_tpl.get("capability")
-        if capabilityName:
-            if capability.name == capabilityName or capability.isCompatibleType(
-                capabilityName
-            ):
-                return True
-        if self.relationship:
-            t = self.relationship.toscaEntityTemplate.type_definition
-            return any(capability.isCompatibleType(cap) for cap in t.valid_target_types)
-        return False
-
 
 class CapabilitySpec(EntitySpec):
     def __init__(self, parent=None, capability=None):
@@ -676,6 +782,10 @@ class CapabilitySpec(EntitySpec):
         self._relationships = None
         self._defaultRelationships = None
 
+    @property
+    def parent(self):
+        return self.parentNode
+
     @property
     def artifacts(self):
         return self.parentNode.artifacts
@@ -713,11 +823,6 @@ class CapabilitySpec(EntitySpec):
         ]
 
 
-# XXX
-# class GroupSpec(EntitySpec):
-#  getNodeTemplates() getInstances(), getChildren()
-
-
 class TopologySpec(EntitySpec):
     # has attributes: tosca_id, tosca_name, state, (3.4.1 Node States p.61)
     def __init__(self, spec=None, inputs=None):
@@ -741,6 +846,7 @@ class TopologySpec(EntitySpec):
         self.properties = {}
         self.defaultAttributes = {}
         self.attributeDefs = {}
+        self.capabilities = []
 
     def getInterfaces(self):
         # doesn't have any interfaces
@@ -754,6 +860,24 @@ class TopologySpec(EntitySpec):
     def isCompatibleType(self, typeStr):
         return False
 
+    @property
+    def baseDir(self):
+        return self.spec.baseDir
+
+    def __reflookup__(self, key):
+        """Make attributes available to expressions"""
+        try:
+            return super(TopologySpec, self).__reflookup__(key)
+        except KeyError:
+            nodeTemplates = self.spec.nodeTemplates
+            nodeSpec = nodeTemplates.get(key)
+            if nodeSpec:
+                return nodeSpec
+            matches = [n for n in nodeTemplates.values() if n.isCompatibleType(key)]
+            if not matches:
+                raise KeyError(key)
+            return matches
+
 
 class Workflow(object):
     def __init__(self, workflow):
@@ -815,6 +939,13 @@ class Artifact(EntitySpec):
     def file(self):
         return self.toscaEntityTemplate.file
 
+    @property
+    def baseDir(self):
+        if self.toscaEntityTemplate._source:
+            return getBaseDir(self.toscaEntityTemplate._source)
+        else:
+            return super(Artifact, self).baseDir
+
     def getPath(self, resolver=None):
         return self.getPathAndFragment(resolver)[0]
 
@@ -843,6 +974,23 @@ class GroupSpec(EntitySpec):
         EntitySpec.__init__(self, template, spec)
         self.members = template.members
 
+    # XXX getNodeTemplates() getInstances(), getChildren()
+
     @property
     def memberGroups(self):
         return [self.spec.groups[m] for m in self.members if m in self.spec.groups]
+
+    @property
+    def policies(self):
+        if not self.spec:
+            return
+        for p in self.spec.policies.values():
+            if p.toscaEntityTemplate.targets_type == "groups":
+                if self.name in p.members:
+                    yield p
+
+
+class PolicySpec(EntitySpec):
+    def __init__(self, template, spec):
+        EntitySpec.__init__(self, template, spec)
+        self.members = set(template.targets_list)
