@@ -1,4 +1,4 @@
-TOSCA_VERSION = 'tosca_simple_unfurl_1_0_0'
+TOSCA_VERSION = "tosca_simple_unfurl_1_0_0"
 
 # loaded by toscaparser extention manager using entrypoint declared in setup.cfg
 class plugindef_1_0_0(object):
@@ -6,4 +6,4 @@ class plugindef_1_0_0(object):
 
     DEFS_FILE = "tosca-ext.yaml"
 
-    SECTIONS = ('metadata')
+    SECTIONS = ("metadata", "decorators")
